package modele.calculs;

import java.util.ArrayList;
import java.util.Observable;

import static java.lang.Thread.*;

public class Ordonnanceur extends Observable implements Runnable {
    private Jeu jeu;
    private ArrayList<RealisateurDeDeplacement> lstDeplacements = new ArrayList<RealisateurDeDeplacement>();
    private ArrayList<RealisateurDeDeplacement> buffer_lstDeplacements = new ArrayList<RealisateurDeDeplacement>();
    private long pause;
    private boolean toClear = false;

    public void add(RealisateurDeDeplacement deplacement) {
        buffer_lstDeplacements.add(deplacement);
    }

    public void clear() {
        toClear = true;
    }

    private void addBuffer() {
        for (RealisateurDeDeplacement r : buffer_lstDeplacements)
            lstDeplacements.add(r);
        buffer_lstDeplacements.clear();
        forceUpdate();
    }

    public Ordonnanceur(Jeu _jeu) {
        jeu = _jeu;
    }

    public void start(long _pause) {
        pause = _pause;
        new Thread(this).start();
    }

    public void forceUpdate() {
        setChanged();
        notifyObservers();
    }

    @Override
    public void run() {
        boolean update = false;

        while(true) {
            switch (jeu.getState()) {
                case 0:
                    for (RealisateurDeDeplacement d : lstDeplacements) {
                        if (d.realiserDeplacement())
                            update = true;
                    }

                    ControleMenu.getInstance().resetDirection();
                    break;
                case 2:
                    jeu.resetCmptDepl();
                    for (RealisateurDeDeplacement d : lstDeplacements) {
                        if (d.realiserDeplacement())
                            update = true;
                    }

                    Controle4Directions.getInstance().resetDirection();
                    Colonne.resetDirection();
                    break;
            }

            if (update) {
                setChanged();
                notifyObservers();
            }

            if(toClear) {lstDeplacements.clear();toClear = false;}

            addBuffer();

            try {
                sleep(pause);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

    }
}
